#include <jabber/jlib.h>
#include <jabber/etherx.h>

#define NSCHECK(x,n) (j_strcmp(xmlnode_get_attrib(x,"xmlns"),n) == 0)

#define NS_CLIENT "jabber:client"
#define NS_SERVER "jabber:server"

#define CMD_SET 0
#define CMD_GET 1
#define CMD_CHECK 2
#define CMD_DEL 3
#define CMD_INIT 4

/* mapi calls return a flag */
#define M_NONE -1
#define M_OK 0
#define M_IGNORE_PHASE 1
#define M_IGNORE_SESSION 2

/* mapi.result, was it handled? */
#define M_DEFAULT 0
#define M_HANDLED 1
#define M_ERR 2

/* mapi.phase */
#define M_PHASE_NONE 0
#define M_PHASE_AUTH 1
#define M_PHASE_SESSION_ON 2
#define M_PHASE_SESSION_OFF 3
#define M_PHASE_MESSAGE_IN 4
#define M_PHASE_MESSAGE_OUT 5
#define M_PHASE_PRESENCE_IN 6
#define M_PHASE_PRESENCE_OUT 7
#define M_PHASE_IQ 8
#define M_PHASE_IQ_HANDLE 9
#define M_PHASE_CONFIG 10
#define M_PHASE_ROSTER 11
#define M_PHASE_UNKNOWN 12
#define M_PHASE_SERVER 13
#define M_PHASE_INFO 14
#define M_PHASE_FREE 15

#define RESOURCE_SESSION 0
#define RESOURCE_HIDDEN 1
#define RESOURCE_SHADOW 2

extern xmlnode config;
char *config_names(int cmd, char *name);
char *config_map(int cmd, char *type, char *host, xmlnode x);
void config_service(xmlnode x);
void config_startup(void);
void config_listen(xmlnode x);
void config_handle(xmlnode x);

void etherx_receive(epacket e);

typedef struct resource_struct
{
    int type;
    char *user;
    char *res;
    pth_msgport_t handler;
    void *aux;
    struct resource_struct *next;
} _resource, *resource;

resource resource_create(int type, char *user, char *res, pth_msgport_t handler);
resource resource_lookup(char *user, char *res);
void resource_free(resource r);
void deliver(jpacket p);

/* global list of per-user data & settings */
typedef struct user_struct
{
    char *user;
    int priority;
    resource primary;
    xmlnode roster;
    jid subscribers;
    struct user_struct *next;
} *udata;

extern udata users;
udata user_data(char *user);


typedef struct remote_struct
{
    char *remote;
    int o_default;
    int o_presence;
    udata u;
} *rdata;

void hidden_create(char *user, char *res, char *remote);
void shadow_create(char *user, char *res, char *remote, int o_default, int o_presence);
void hidden_kill(char *user, char *res);
void shadow_kill(char *user, char *res);


typedef void (*session_onSend)(thread t, jpacket p);
 
typedef struct session_struct
{
    resource r;
    pth_msgport_t mp_from;
    pth_msgport_t mp_to;
    session_onSend send_evt;
    void *send_evt_data;
    jid id;
    struct session_struct *next;
    int exit_flag;
    int in_count;
    int out_count;
    pool p;
    xmlnode presence;
    udata u;
} *session;

thread session_create(jid owner, session_onSend evt, void *evt_data);
void *session_main(void *arg);
void session_kill(char *user, char *resource, char *reason);
void session_end(session s, char *reason);
session session_primary(char *user);
int session_from_handle(session s, jpacket p);
int session_to_handle(session s, jpacket p);
void session_roster_handle(session s, jpacket p);

void roster_load(char *user, char *group, char *item, char *name); /* builds the roster for the user */
xmlnode roster_get(char *user);
void subscriber_rebuild(char *user);

void conn_connect(thread t, int sock, struct sockaddr_in sa);
void conn_negotiate(thread t, xmlnode in, int type);
xmlstream_onNode conn_namespace(int cmd, char *ns, xmlstream_onNode handler);

void *remote_main(void *arg);
void *server_main(void *arg);
void server_info(jpacket p);
void *external_main(void *arg);
void *unknown_main(void *arg);

typedef struct {
    pth_message_t head; /* the standard pth message header */
    jpacket p;
} _jpq, *jpq;

void psend(pth_msgport_t mp, jpacket p); /* sends p to a pth message port */
void *dso_load(char *so, char *init, void *arg);

typedef struct mapi_struct
{
    int phase;
    session s;
    jpacket pkt;
    pool p;
    int result;
    int cmd;
    char *user;
    char *resource;

    /* other objects passed to modules for all dispatch calls */

    /* auth */
    char *password;
    char *auth_error;

    /* roster */
    char *group;
    char *item;
    char *name;

} *mapi;

mapi mapi_new(int phase, pool p, session s, jpacket pkt);
void module_dispatch(mapi m);
void modules_init(xmlnode x);

int presence_priority(xmlnode x);
int presence_check(int cmd, char *id);

void logger_init(xmlnode x);
void logger_session(session s);
void logger_registration(char *host, char *user);

void shutdown_push(void (*end)());
void shutdown_now();

xmlnode message_error(xmlnode x, char *errstr);
xmlnode iq_reply(xmlnode x);
void bounce(jpacket p, char *reason);
void xdelay(xmlnode msg, char *reason);

void server_agents(jpacket p);
void server_agent(char *id);
