#include "jserver.h"

xmlnode config = NULL;

/* start all system threads here, init any globals */
void config_startup(void)
{
    pth_attr_t attr;

    jdebug(ZONE,"JSERVER STARTUP!");

    attr = pth_attr_new();
    pth_attr_set(attr, PTH_ATTR_JOINABLE, FALSE);
    pth_attr_set(attr, PTH_ATTR_STACK_SIZE, jlib_stack_default);

    pth_spawn(attr, external_main, NULL); /* the external thread */
    pth_spawn(attr, unknown_main, NULL); /* the thread handling all unknown resources */
    pth_spawn(attr, server_main, NULL); /* all traffic to server resources */
    pth_spawn(attr, remote_main, NULL); /* handles remote resources */

    pth_attr_destroy(attr);
    pth_yield(NULL); /* let pth start the thread */
}

/* this is where everything is configured */
void config_handle(xmlnode x)
{
    xmlnode next, l, m;
    int listen_flag = 0;
    static int configured = 0;

    jdebug(ZONE,"config_handle");
    xmlnode_free(config); /* re-config */
    config = xmlnode_dup(x);

    /* we got the chunk from the config file, 
    make sure we init everything depending on config reguardless if they were configured  */
    next = xmlnode_get_firstchild(x);
    l = m = NULL;
    while(next != NULL)
    {
        if(j_strcmp(xmlnode_get_name(next),"name") == 0)
            config_names(CMD_SET,xmlnode_get_data(xmlnode_get_firstchild(next)));
        if(j_strcmp(xmlnode_get_name(next),"map") == 0)
            config_map(CMD_SET,NULL,NULL,next);
        if(j_strcmp(xmlnode_get_name(next),"listen") == 0)
        {
            listen_flag = 1;
            config_listen(next);
        }
        if(j_strcmp(xmlnode_get_name(next),"service") == 0 && !configured)
            config_service(next);
        if(j_strcmp(xmlnode_get_name(next),"log") == 0)
            l = next;
        if(j_strcmp(xmlnode_get_name(next),"modules") == 0)
            m = next;
        next = xmlnode_get_nextsibling(next);
    }

    if(!listen_flag && !configured)
        config_listen(NULL); /* make sure we start listening even if it wasn't configure */

    logger_init(l);
    modules_init(m);

    ++configured;
}

struct names
{
    char *name;
    struct names *next;
};

/* this is a special internal name resolver for the dns name(s) of this transport */
char *config_names(int cmd, char *name)
{
    static struct names *n = NULL, *get = NULL;
    struct names *new;
    
    switch(cmd)
    {
        case CMD_SET: /* only intended to be run at config time */
            if(config_names(CMD_CHECK,name) != NULL || name == NULL)
                return NULL;
            etherx_alias(name);
            new = malloc(sizeof(struct names));
            new->name = j_strdup(name);
            new->next = n;
            n = new;
            break;
        case CMD_CHECK:
            new = n;
            while(new != NULL)
            {
                if(j_strcmp(name,new->name) == 0)
                    return new->name;
                new = new->next;
            }
            break;
        case CMD_GET: /* loops through name list, returns NULL at end */
            if(get == NULL)
                get = n;
            else
                get = get->next;
            if(get != NULL)
                return get->name;
            break;
    }
    return NULL;
}

struct map
{
    char *id;
    char *type;
    char *host;
    struct map *next;
};

/* 
this is a container to store the configured maps
*/
char *config_map(int cmd, char *type, char *host, xmlnode x)
{
    static struct map *m = NULL;
    struct map *next;

    switch(cmd)
    {
        case CMD_SET: /* extract the transport n-fo from the config */
            next = malloc(sizeof(struct map));
            next->type = j_strdup(xmlnode_get_attrib(x,"type"));
            next->host = j_strdup(xmlnode_get_attrib(x,"host"));
            next->id = j_strdup(xmlnode_get_data(xmlnode_get_firstchild(x)));
            next->next = m;
            m = next;
            break;
        case CMD_CHECK: /* find the given id and return the name */
            next = m;
            while(next != NULL)
            {
                if(j_strcmp(next->type, type) == 0)
                    return next->id;
                if(j_strcmp(next->host, host) == 0)
                    return next->id;
                next = next->next;
            }
            break;
    }

    return NULL;
}

void config_service(xmlnode x)
{
    /* dso load the service and initialize it */
    char *dso;

    dso = xmlnode_get_attrib(x, "dso");
    dso_load(dso, "init", (void *)x);
}

/* start the listen thread(s) based on the configuration */
void config_listen(xmlnode x)
{
    char *ip;
    int port;
 
    ip = xmlnode_get_data(xmlnode_get_firstchild(x));
    if(xmlnode_get_attrib(x, "port") == NULL || sscanf(xmlnode_get_attrib(x, "port"), "%d", &port) == 0)
        port = 5222;
 
    /* during reconfig, new listens will be added, existing ones will err when they try to listen and just fail silently */
    if(tlisten_new(port, ip, conn_connect, NULL) == NULL)
        jerror("jserver","failed to listen on %s:%d",ip,port);
    else
        jdebug(ZONE,"listening on %s:%d\n",ip,port);
}
