#include "jserver.h"

void etherx_receive(epacket e)
{
    /* handles incoming packets from etherx, we get to free the mem */
    jpacket p;

    if(e == NULL)
    {
        config_startup();
        return;
    }

    switch(e->type)
    {
        case EPACKET_NORMAL:
            jdebug(ZONE,"EXTERNAL: got epacket from[%s]: %s",e->from, xmlnode2str(e->x));
            xmlnode_hide_attrib(e->x, "etherx:to");
            xmlnode_hide_attrib(e->x, "etherx:from");
            p = jpacket_new(e->x);
            deliver(p);
            break;
        case EPACKET_ERROR:
            jdebug(ZONE,"EXTERNAL: got error epacket bounce to %s reason %s",xmlnode_get_attrib(e->x,"etherx:from"),xmlnode_get_attrib(e->x,"etherx:error"));
            xmlnode_hide_attrib(e->x, "etherx:to");
            xmlnode_hide_attrib(e->x, "etherx:from");
            xmlnode_hide_attrib(e->x, "etherx:error");
            message_error(e->x,"Server Connection Failed");
            deliver(jpacket_new(e->x));
            break;
        case EPACKET_REGISTRY:
            config_handle(e->x);
            xmlnode_free(e->x);
            break;
    }
}

void *external_main(void *arg)
{
    /* thread that waits for messages intended to be sent externally, calls etherx_write */
    /* service phase for externally bound packets? */
    /* use transports and jid->type, deliver to transport for type */
    pth_event_t ev;
    pth_msgport_t mp;
    jpq q;
    epacket e;
    char *server;
    
    jdebug(ZONE,"THREAD:EXTERNAL starting");

    mp = pth_msgport_create("external");
    ev = pth_event(PTH_EVENT_MSG,mp);

    while(1)
    {
        pth_wait(ev);
        while((q = (jpq)pth_msgport_get(mp)) != NULL)
        {
            if(q->p->to == NULL || q->p->from == NULL)
            {
                jdebug(ZONE,"THREAD:EXTERNAL invalid recipients for external delivery");
                xmlnode_free(q->p->x);
                break;
            }
            jdebug(ZONE,"THREAD:EXTERNAL received a message!");

            /* remap the address */
            server = q->p->to->server;
            if(config_map(CMD_CHECK,q->p->to->type,server,NULL) != NULL)
                server = config_map(CMD_CHECK,NULL,server,NULL);

            e = epacket_new(q->p->x, server, q->p->from->server, NS_SERVER);
            etherx_send(e); /* will consume/free x */
        }
    }
    
    pth_event_free(ev,PTH_FREE_ALL);
    pth_msgport_destroy(mp);
}
