#include "jserver.h"

void hidden_create(char *user, char *res, char *remote)
{
    resource r;
    rdata rd;

    if(user == NULL || res == NULL || remote == NULL)
        return;

    hidden_kill(user, res);
    r = resource_create(RESOURCE_HIDDEN, user, res, pth_msgport_find("remote"));
    rd = malloc(sizeof(rdata));
    rd->remote = strdup(remote);
    r->aux = (void *)rd;
}

void shadow_create(char *user, char *res, char *remote, int o_default, int o_presence)
{
    resource r;
    rdata rd;

    if(user == NULL || res == NULL || remote == NULL)
        return;

    shadow_kill(user, res);
    r = resource_create(RESOURCE_SHADOW, user, res, pth_msgport_find("remote"));
    rd = malloc(sizeof(rdata));
    rd->remote = strdup(remote);
    rd->o_default = o_default;
    rd->o_presence = o_presence;
    if(o_default)
        rd->u = user_data(user);
    r->aux = (void *)rd;
}

void hidden_kill(char *user, char *res)
{
    resource r;
    rdata rd;

    r = resource_lookup(user,res);
    if(r == NULL)
        return;

    rd = (rdata)(r->aux);
    free(rd->remote);
    free(rd);
    resource_free(r);
}

void shadow_kill(char *user, char *res)
{
    hidden_kill(user,res);
}


/* thread that handles data for remote resources */
void *remote_main(void *arg)
{
    pth_event_t ev;
    pth_msgport_t mp;
    jpq q;
    resource r;
    rdata rd;
    jpacket p;
    xmlnode presence;
    jid to_id;
    mapi m;
    int flag;

    jdebug(ZONE,"THREAD:REMOTE startup!");    

    mp = pth_msgport_create("remote");
    ev = pth_event(PTH_EVENT_MSG,mp);

    while(1)
    {
        pth_wait(ev);
        while((q = (jpq)pth_msgport_get(mp)) != NULL)
        {
            jdebug(ZONE,"THREAD:REMOTE received a message!");

            p = q->p;
            r = resource_lookup(p->to->user, p->to->resource);
            if(r == NULL)
            {
                xmlnode_free(p->x);
                continue;
            }
            rd = (rdata)(r->aux);

            if(r->type == RESOURCE_HIDDEN)
            {
                if(j_strcmp(p->from->user, p->to->user) == 0)
                { /* from this user to their hidden resource, forward on */
                    xmlnode_put_attrib(p->x,"to",rd->remote);
                }else{ /* to this user, from the remote resource, fudge and broadcast/forward to the user */
                    xmlnode_put_attrib(p->x,"from",jid_full(p->to));
                    jid_set(q->p->to,NULL,JID_RESOURCE);
                    xmlnode_put_attrib(p->x,"to",jid_full(p->to));
                }
                deliver(jpacket_new(p->x));
                continue;
            }

            if(r->type == RESOURCE_SHADOW)
            {
                if(j_strcmp(jid_full(p->from),rd->remote) != 0)
                { /* from anyone to the shadow resource, forward on */
                    xmlnode_put_attrib(p->x,"to",rd->remote);
                    deliver(jpacket_new(p->x));
                    continue;
                }

                /* from the remote resource */
                xmlnode_put_attrib(p->x,"from",jid_full(p->to));
                if(p->type != JPACKET_PRESENCE)
                { /* anything other than presence packets just get forwarded straight up */
                    jid_set(q->p->to,NULL,JID_RESOURCE);
                    xmlnode_put_attrib(p->x,"to",jid_full(p->to));
                    deliver(jpacket_new(p->x));
                    continue;
                }

                if(rd->o_default)
                {
                    /* ha ha, we're better than you, give us default messages! */
                    if(presence_priority(p->x) >= rd->u->priority)
                    {
                        rd->u->priority = presence_priority(p->x);
                        rd->u->primary = r;
                    }else if(rd->u->primary == r){ /* we were default, but we're lower now :( */
                        rd->u->priority = presence_priority(p->x);
                    }
                }

                if(rd->o_presence)
                {
                    /* force broadcast to our other resources */
                    presence = xmlnode_dup(p->x);
                    jid_set(p->to,NULL,JID_RESOURCE);
                    xmlnode_put_attrib(presence,"to",jid_full(p->to));
                    deliver(jpacket_new(presence));

                    m = mapi_new(M_PHASE_PRESENCE_OUT,p->p,NULL,p);
                    m->user = r->user;
                    m->resource = r->res;
                    module_dispatch(m);
                    if(m->result == M_DEFAULT)
                    { /* module didn't handle it, we'll use our internal roster */
                        to_id = rd->u->subscribers;
                        flag = j_strcmp(xmlnode_get_attrib(p->x,"type"),"available");
                        if(flag != 0)
                            flag = j_strcmp(xmlnode_get_attrib(p->x,"type"),"unavailable");
                        while(to_id != NULL)
                        {
                            if(flag != 0 && presence_check(CMD_GET,jid_full(to_id)) == 0)
                            { /* if this isn't an [un]available type, only send to those we know are available */
                                to_id = to_id->next;
                                continue;
                            }
                            jid_set(to_id,NULL,JID_RESOURCE);
                            presence = xmlnode_dup(p->x);
                            xmlnode_put_attrib(presence,"to",jid_full(to_id));
                            deliver(jpacket_new(presence));
                            to_id = to_id->next;
                        }
                    }
                }
            }

            xmlnode_free(p->x);
            continue;
        }
    }
    
    pth_event_free(ev,PTH_FREE_ALL);
    pth_msgport_destroy(mp);
}


