#include "jserver.h"

int presence_priority(xmlnode x)
{
    char *str;
    int ret = 0;

    if(x == NULL)
        return ret;

    x = xmlnode_get_tag(x,"priority");
    if(x == NULL)
        return ret;

    str = xmlnode_get_data(xmlnode_get_firstchild(x));
    if(str == NULL)
        return ret;

    return atoi(str);
}

struct pcheck_struct
{
    char *id;
    struct pcheck_struct *next;
};

/* simple little bit flipper utility tracking id's availability server-wide */
int presence_check(int cmd, char *id)
{
    static struct pcheck_struct *list = NULL;
    struct pcheck_struct *next, *prev;

    if(id == NULL)
        return 0;

    prev = next = list;
    while(next != NULL)
    {
        if(strcmp(next->id,id) == 0)
            break;
        prev = next;
        next = next->next;
    }

    if(cmd == CMD_SET)
    {
        if(next == NULL)
        {
            next = malloc(sizeof(struct pcheck_struct));
            next->id = strdup(id);
            next->next = list;
            list = next;
        }else{
            free(next->id);
            next->id = strdup(id);
        }
    }

    if(cmd == CMD_DEL && next != NULL)
    {
        if(list == next)
            list = list->next;
        else
            prev->next = next->next;
        free(next->id);
        free(next);
        next = NULL;
    }

    if(next == NULL)
        return 0;
    else
        return 1;
}

udata users = NULL;

/* you can't pass this NULL! */
udata user_data(char *user)
{
    udata next;

    next = users;
    while(next != NULL)
    {
        if(j_strcmp(next->user, user) == 0)
            return next;
        next = next->next;
    }

    next = malloc(sizeof(struct user_struct));
    memset(next, '\0', sizeof(struct user_struct));
    next->user = strdup(user);
    next->next = users;
    next->priority = 0;
    next->primary = NULL;
    next->roster = NULL;
    next->subscribers = NULL;
    users = next;
    return next;
}

struct shutdown_struct
{
    void (*f)();
    struct shutdown_struct *next;
};

struct shutdown_struct *shutdown_list = NULL;

void shutdown_push(void (*end)())
{
    struct shutdown_struct *list;

    if(end == NULL)
        return;

    list = malloc(sizeof(struct shutdown_struct));
    list->next = shutdown_list;
    list->f = end;
    shutdown_list = list;
}

void shutdown_now()
{
    while(shutdown_list != NULL)
    {
        (shutdown_list->f)();
        shutdown_list = shutdown_list->next;
    }
}

/* morph the xmlnode from a message into an error */
xmlnode message_error(xmlnode x, char *errstr)
{
    char *sub_data, *to, *from;
    xmlnode sub;

    if(x == NULL)
        return NULL;

    if(errstr == NULL)
        errstr = pstrdup(xmlnode_pool(x),"Message Delivery Failure");

    sub = xmlnode_get_tag(x,"subject");
    sub_data = xmlnode_get_data(xmlnode_get_firstchild(sub));
    to = xmlnode_get_attrib(x,"to");
    from = xmlnode_get_attrib(x,"from");

    xmlnode_put_attrib(x,"type","error");

    xmlnode_hide(xmlnode_get_firstchild(sub));
    if(sub == NULL)
        sub = xmlnode_insert_tag(x,"subject");

    xmlnode_insert_cdata(sub,errstr,strlen(errstr));
    if(sub_data != NULL || to != NULL)
    {
        xmlnode_insert_cdata(sub," [",2);
        if(to != NULL)
        {
            xmlnode_insert_cdata(sub,"to: ",4);
            xmlnode_insert_cdata(sub,to,strlen(to));
            xmlnode_insert_cdata(sub," ",1);
        }
        if(sub_data != NULL)
        {
            xmlnode_insert_cdata(sub,"was: ",5);
            xmlnode_insert_cdata(sub,sub_data,strlen(sub_data));
            xmlnode_insert_cdata(sub," ",1);
        }
        xmlnode_insert_cdata(sub,"]",1);
    }

    if(from != NULL)
    {
        xmlnode_put_attrib(x,"to",from);
        xmlnode_hide_attrib(x,"from");
    }

    return x;
}

xmlnode iq_reply(xmlnode x)
{
    char *to, *from;

    to = xmlnode_get_attrib(x,"to");
    from = xmlnode_get_attrib(x,"from");

    xmlnode_put_attrib(x,"type","result");

    if(to != NULL)
        xmlnode_put_attrib(x,"from",to);
    if(from != NULL)
        xmlnode_put_attrib(x,"to",from);

    return x;
}

char *gen_timestamp(void)
{
	time_t t;
	struct tm *new_time;
	static char timestamp[18];
	int ret;
	
	t = time(NULL);

	if(t == (time_t)-1)
		return NULL;
	new_time = gmtime(&t);

	ret = snprintf(timestamp, 17, "%d%d%dT%d:%d:%d", 1900+new_time->tm_year,
					new_time->tm_mon+1, new_time->tm_mday, new_time->tm_hour,
					new_time->tm_min, new_time->tm_sec);

	if(ret == -1)
		return NULL;

	return timestamp;
}

void bounce(jpacket p, char *reason)
{
    if(p->type == JPACKET_MESSAGE && j_strcmp(xmlnode_get_attrib(p->x,"type"),"error") != 0)
        deliver(jpacket_new(message_error(p->x,reason)));
    else if(p->type == JPACKET_IQ && j_strcmp(xmlnode_get_attrib(p->x,"type"),"result") != 0)
        deliver(jpacket_new(iq_reply(p->x)));
    else
        xmlnode_free(p->x);
}

void xdelay(xmlnode msg, char *reason)
{
    xmlnode delay;

    delay = xmlnode_insert_tag(msg,"x");
    xmlnode_put_attrib(delay,"xmlns","jabber:x:delay");
    xmlnode_put_attrib(delay,"from",xmlnode_get_attrib(msg,"to"));
    xmlnode_put_attrib(delay,"stamp",gen_timestamp());
    if(reason != NULL)
        xmlnode_insert_cdata(delay,reason,strlen(reason));
}
